%-------------
% LICENSE
%-------------

%This program is free software; you can redistribute it and/or modify it
%under the terms of the GNU General Public License version 3 as published
%by the Free Software Foundation (LGPL may be granted upon request). This 
%program is distributed in the hope that it will be useful, but without any 
%warranty; without even the implied warranty of merchantability or fitness for a particular purpose. See the GNU General Public License for more details.



% Lauch script with sim = runIOSim(n) where n is one of the following
% choices:
% mode 0:   robot, real time, CC to OC switch
% mode 1:   sim, manual, real time, CC to OC switch (equiv. to robot)
% mode 1.5: manual, OC only
% mode 2:   CC only, auto, accelerated
% mode 2.5: CC only, auto, accelerated: brief stimuli, late rewards     
% mode 3:   OC only, auto, accelerated

classdef runIOSim < handle
    
    properties
        
        par;                             % all fixed parameters that do not change during execution
        
        stepsToReward                   = -1;
        CCstimuliDuration;              % for simulated IO 
        touched                         = 0;
        touchReceived                   = 0;
        punished                        = 0;
        learningMode                    = 1;
        icubphrases;
        trackingStimulus                = -1;
        currentOCStimulus               = 0;
        readyForNextActionCountDown     = 0;
        newStimulus                 = 0;
        mode                        = 0;    % when the mode is 0 stimuli and rewards are given automatically
        
        CSstimuli;                          % vector of input stimuli for CC
        CSstimuliOld;                       % used to determined when a stimulus end
        OCstimuli;                          % vector of input stimuli for OC
        
        netCC;                              % classical conditioning network
        netOC;                              % operant conditioning network
        
        USsounds;
        CSsounds;
        Actionsounds;
        Miscsounds;
        log;
        fig;
        runbutton;
        currentStep;
        chargingAction                      = 0;
        stimulusEpisode                     = 0;
        futureReward                        = 0;
        plottingcounter                     = 0;
    end
    
    methods
        % mode 0: robot, real time, CC to OC swith
        % mode 1: sim, manual, real time, CC to OC switch (equiv. to robot)
        % mode 1.5: manual, OC only
        % mode 2: CC only, auto, accelerated
        % mode 2.5:  CC only, auto, accelerated: brief stimuli, late rewards
        % mode 3: OC only, auto, accelerated
        
        function app = runIOSim(mode)
            % #############################################################
            %% Parameter settings 
            %##############################################################
            app.par.simTime                     = 10800;                    % max duration of the experiment in seconds
            app.par.samplingTime                = 0.20;                    % sampling time in seconds
            app.par.seed                        = 3114;                    % random seed for pseudo number generator
            
            app.par.nrCCStimuli                 = 9;                       % number of stimuli in the CC experiment
            app.par.nrCCOutputs                 = 1;                       % number of outputs in the CC experiment
            
            app.par.nrOCStimuli                 = 2;                       % number of stimuli in the OC experiment
            app.par.nrOCActions                 = 8;                       % number of outputs in the OC experiment
                        
            app.par.OCactionReward              = 2;                       % intensity of modulation for correct answer in OC experiment
            app.par.actionPunishment            = -0.5 * app.par.OCactionReward;
            app.par.minimalIntervalBetweenActions = 6;                     
            
            app.par.smileDuration               = 2;
            app.par.actionDuramtion             = 2;
            app.par.maxRewardDelay              = 5;     % what's the maximum delay between seeing CC-CS and receving the reward?
            app.par.minRewardDelay              = 0;
            app.par.CSabsenceDetectedAfter      = 10;
            app.par.probSwithcingToOCperSec     = 0.05;
            app.par.colourShift                 = 0;
            
            app.mode = mode;
            app.CCstimuliDuration =  zeros(app.par.nrCCStimuli,1);
            
            app.netCC = neunet(app.par.seed, app.par.nrCCStimuli, app.par.nrCCOutputs, app.par.samplingTime, app.par.simTime, 1);
            app.netOC = neunet(app.par.seed, app.par.nrOCStimuli, app.par.nrOCActions, app.par.samplingTime, app.par.simTime, 2);

            
            if mode < 2
                % Manual mode
                app.par.probStimuliPerSecond        = 0.00;
                app.par.maxStimulusDuration         = 600;
                app.par.rewardRefractoryTime        = 0;
                app.par.OCactionReward              = 1;                       % intensity of modulation for correct answer in OC experiment

            else
                % Automatic mode ( simulate the poisson process of people arriving )
                app.par.probStimuliPerSecond        = 0.0015;
                app.par.maxStimulusDuration         = 30;
                app.par.minStimulusDuration         = 3;
                app.par.rewardRefractoryTime        = 4;
                
                % case of reward after stimulus is gone
                if mode == 2.5
                    app.par.minStimulusDuration         = 1;
                    app.par.maxStimulusDuration         = 2;
                    app.par.probStimuliPerSecond        = 0.005;
                    app.netCC.par.maxModulation         = 0.07;
                end
            end
            
            %##############################################################
            % Execution settings
            if mode >= 2 % auto CC simulation
                app.par.rewardingStimulus           = 5;
                answer = input(['Stimulus for tutor? [default ' num2str(app.par.rewardingStimulus) ']: ']);
                if ~isempty(answer)
                    if answer > 0 && answer <= app.par.nrCCStimuli
                        app.par.rewardingStimulus = answer;
                    else
                        error(['The rewarding stimulus must be between 1 and ' num2str(app.par.nrCCStimuli)]);
                    end
                end
            end
            
            app.par.refreshEvery                = 1;
            
            % #############################################################
            % runtime variables instantiation
            disp('Classical and Operant Conditioning - Simulation of input-output flow');
             
               
                         
            app.createWindow();
            set(app.fig.panel,'KeyPressFcn',@app.keypressed);
            set(app.fig.OCpanel,'Visible','off');
            
            path = '../data/';
            if ~isdir(path)
                path = '../../data/';
            end
            if ~isdir(path)
                path = './';
            end
            app.icubphrases = load([path filesep 'icub-phrases.mat']);
            for i = 1:8
                app.USsounds{i} = audioplayer(app.icubphrases.phrasesUS(i,:),44100);
            end
            for i = 1:3
                app.CSsounds{i} = audioplayer(app.icubphrases.phrasesCS(i,:),44100);
            end
            for i =1:9
                app.Actionsounds{i} = audioplayer(app.icubphrases.phrasesAction(i,:),44100);
            end
            app.Miscsounds{1} = audioplayer(app.icubphrases.phrasesMisc(1,:),44100);
            
            
            app.log.idleTime = zeros(app.par.simTime / app.par.samplingTime, 1);
            app.CSstimuli = zeros(app.par.nrCCStimuli,1);
            app.CSstimuliOld = zeros(app.par.nrCCStimuli,1);
            app.OCstimuli = zeros(app.par.nrOCStimuli,1);
        end
        
        function running(app,src,evn)
            if get(src, 'Value')
            CSabsentFor = 0;
            sayColour = [];
            steps = 2;
            
            app.startCC();
            
            while get(src, 'Value') && steps < app.par.simTime/app.par.samplingTime
                
                %tic;  % measuring time for real-time computation 
                t1 = clock;
                
                app.currentStep = steps;
                
                % READING INPUT HERE ######################################
                app.readStimuli();
                
                % selection of run-modes (manual / auto / CC / OC)
                switch app.mode
                    case 1.5,
                        app.learningMode = 2;
                        app.trackingStimulus = 1;
                    case 2,
                        app.learningMode = 1;
                    case 3
                        app.learningMode = 2;
                        app.trackingStimulus = app.par.rewardingStimulus;
                        app.CSstimuli(app.par.rewardingStimulus) = 1;
                        % setting on the OC stimulus
                        if mod(app.currentStep,round(20/app.par.samplingTime)) == 1
                            app.stimulusEpisode = app.stimulusEpisode + 1;
                            app.currentOCStimulus = mod(app.stimulusEpisode, app.par.nrOCStimuli) + 1;
                            app.OCstimuli(app.currentOCStimulus) = 1;
                        end
                end
                
                switch app.learningMode
                    %######################################################
                    case 1,
                        % Classical Conditioning (learningMode == 1)
                        
                        % CASE OF REWARD WHILE STIMULUS
                        if app.mode == 2 && ...
                            app.CSstimuli(app.par.rewardingStimulus) == 1 && app.stepsToReward < -app.par.rewardRefractoryTime / app.par.samplingTime
                            app.stepsToReward = app.netCC.myrandi(1,1,1,round(app.par.maxRewardDelay/app.par.samplingTime));
                        end
                        % CASE OF REWARD AFTER STIMULUS
                        if app.mode == 2.5 
                            if app.CSstimuli(app.par.rewardingStimulus) == 0 ...
                                && app.CSstimuliOld(app.par.rewardingStimulus) == 1 ...
                                
                            app.stepsToReward = app.netCC.myrandi(1,1,1,round(app.par.maxRewardDelay/app.par.samplingTime)) + 1;
                            end
                            
                            % The following if introduces CS2 with CS1 to
                            % show that blocking is not implemented in the
                            % current model
                            %if app.currentStep > 20000 && app.CSstimuli(app.par.rewardingStimulus) == 1
                            %     app.CSstimuli(app.par.rewardingStimulus+1) = 1;
                            %end
                        end
                        
                        
                        if app.touchReceived && app.mode <= 1
                            app.stepsToReward = 0;
                        end
                        
                        if app.stepsToReward == 0;
                            reward = 1;
                            app.robotRewardCC();
                            app.startCC();
                        else
                            reward = 0;
                        end
                        
                        % UPDATE NETWORK HERE #############################
                        app.netCC.update(app.CSstimuli,reward);
                        
                        if app.netCC.CS && app.netCC.stimulusToTrack >= 0
                            if app.trackingStimulus < 0 && app.mode < 2
                                app.trackingStimulus = app.netCC.stimulusToTrack;
                                app.startTrackingTutor();
                                %pause(0.5);
                                
                                try
                                   % playblocking(app.CSsounds{app.netCC.myrandi(1,1,1,2)});
                                    play(app.CSsounds{app.netCC.myrandi(1,1,1,2)});
                                end
                                
                                disp(['Starting to track stimulus ' num2str(app.trackingStimulus)]);
                            end
                        end
                        
                        % OC start randomly while following the CS stimulus
                        if app.trackingStimulus >= 0 &&...
                                app.mode < 2 &&...
                                rand(app.netCC.stream,1) < (app.par.probSwithcingToOCperSec * app.par.samplingTime)
                            app.startOC();
                        end
                        
                    %######################################################    
                    case 2,
                        % Operant Conditioning (learning mode == 2)
                        
                        % monitors that tutor is present
                        if app.CSstimuli(app.trackingStimulus) == 0
                            CSabsentFor = CSabsentFor + 1;
                        else
                            CSabsentFor = 0;
                        end
                        
                        % assign punishment
                        if app.punished == 1
                            punishmentReceived = 1;
                            app.punished = 0;
                        else
                            punishmentReceived = 0;
                        end
                        
                        % assign reward if touched and play sound
                        reward = 0;
                        if app.touchReceived == 1
                            app.robotRewardOC();
                            
                            reward = app.par.OCactionReward;
                            if app.mode < 2
                                try
                                    play(app.USsounds{app.netOC.myrandi(1,1,1,5)});
                                end
                            end
                        end
                        
                        % assign reward if punished and play sound
                        if punishmentReceived == 1
                            app.robotPunishmentOC();
                            reward = app.par.actionPunishment;
                            try
                                play(app.USsounds{8});
                                disp(reward)
                            end
                        end
                        
                        if app.mode == 3 && app.stepsToReward == 0;
                            reward = app.futureReward;
                        end
                        
                        % UPDATE NETWORK HERE #################################
                        app.netOC.update(app.OCstimuli, reward);
                        % #####################################################
                        
                        sayColour = find(app.netOC.actions == 1, 1);
                        % if ~isempty(sayColourOld) && isempty(sayColour) && app.readyForNextActionCountDown <= 0
                        if ~isempty(sayColour)
                            app.chargingAction = app.chargingAction + 1;
                        end
                        if app.chargingAction > 1/app.par.samplingTime...
                                && app.readyForNextActionCountDown <= 0
                            app.chargingAction = 0;
                            try
                                play(app.Actionsounds{sayColour});
                                disp(['Colour is ' num2str(sayColour)]);
                            end
                            
                            % in auto mode give delay to reward
                            if app.mode == 3
                                
                               % if app.currentStep < 1800 / app.par.samplingTime
                               %     app.par.colourShift = 0;
                               % else
                               %     app.par.colourShift = 3;
                               % end
                                 
                                if sayColour == app.currentOCStimulus + app.par.colourShift
                                    
                                    app.stepsToReward = round( app.par.maxRewardDelay / app.par.samplingTime * rand(app.netOC.stream,1)) +  1;
                                    app.futureReward = app.par.OCactionReward;
                                    disp(['steps to reward: ' num2str(app.stepsToReward )]);
                                else
                                    app.stepsToReward = round( app.par.maxRewardDelay / app.par.samplingTime * rand(app.netOC.stream,1)) +  1;
                                    app.futureReward = app.par.actionPunishment;
                                    %app.futureReward = 0;
                                end
                            end
                            
                            if app.currentOCStimulus > 0
                                app.OCstimuli(app.currentOCStimulus) = 0;
                                app.currentOCStimulus = 0;
                            end
                            
                            app.readyForNextActionCountDown = app.par.minimalIntervalBetweenActions / app.par.samplingTime;
                        end
                        
                        % CS is absent, return to CC learning mode
                        if app.mode <= 1 && CSabsentFor > app.par.CSabsenceDetectedAfter / app.par.samplingTime
                            CSabsentFor = 0;
                            try
                                %playblocking(app.Miscsounds{1});
                                play(app.Miscsounds{1});
                            catch
                                disp('error in play sound')
                            end
                            app.startCC();
                        end
                end
                
                if mod(steps,app.par.refreshEvery/(app.par.samplingTime)) == 0
                    
                    app.plottingcounter = app.plottingcounter+1;
                    app.refreshGraphs(app.plottingcounter);
                end
                
                app.stepsToReward = app.stepsToReward - 1;
                app.readyForNextActionCountDown = app.readyForNextActionCountDown - 1;
                
                app.log.idleTime(steps) = app.par.samplingTime - etime(clock,t1);
                
                if app.log.idleTime(steps) < 0
                    %beep;
                    disp(['ATTENTION! REAL TIME LOSS OF '  num2str(abs(app.log.idleTime(steps))) ' s']);
                end
                
                % pause the execution for the time left
                if app.mode <= 1 && app.log.idleTime(steps) > 0
                    pause(app.log.idleTime(steps))
                end
                steps = steps + 1;
            end
            else
                %run button released
                app.log.numstimuli = sum(sign(app.netCC.log.stimuliRaw(:,:)),2);
                app.stopRunning();
            end
        end
        
        function readStimuli(app)
            app.touchReceived = app.istouched();
            app.punished = app.ispunished();
            
            % robotic mode
            if app.mode == 0
                app.readRobotStimuli();
            else
                newstimuli = zeros(app.par.nrCCStimuli,1);
                
                if app.newStimulus > 0
                    newstimuli(app.newStimulus) = 1;
                    app.newStimulus = 0;
                end
                
                if app.mode >= 2
                    newstimuli = rand(app.netCC.stream,app.par.nrCCStimuli,1) < app.par.probStimuliPerSecond;
                    app.CCstimuliDuration =  min(...
                        app.CCstimuliDuration + (rand(app.netCC.stream,app.par.nrCCStimuli,1)...
                        .* (app.par.maxStimulusDuration - app.par.minStimulusDuration) + app.par.minStimulusDuration) ...
                        .* newstimuli, app.par.maxStimulusDuration);
                   % if newstimuli(app.par.rewardingStimulus) == 1 && app.mode ~= 3
                   %     app.stepsToReward = app.netCC.myrandi(1,1,1,round(app.par.maxRewardDelay/app.par.samplingTime));
                   % end
                else
                    app.CCstimuliDuration = app.CCstimuliDuration + app.par.maxStimulusDuration .* newstimuli;
                    % remove a stimulus when a the stimulus-key is pressed
                    if ~isempty(find(newstimuli == 1, 1)) && app.CSstimuli(newstimuli == 1) == 1
                        disp('removing');
                        app.CCstimuliDuration(newstimuli == 1) = 0;
                    end
                end
                app.CCstimuliDuration = max(app.CCstimuliDuration - app.par.samplingTime,-app.par.samplingTime);
                
                app.CSstimuliOld = app.CSstimuli;
                app.CSstimuli = (app.CCstimuliDuration > 0);
                
                if app.mode < 2 && app.currentOCStimulus > 0;
                    % this flips it on or off each time the keys 'r' or 'g' are pressed
                    app.OCstimuli(app.currentOCStimulus) = ~app.OCstimuli(app.currentOCStimulus);
                    app.currentOCStimulus = 0;
                end
            end
        end
        
        function robotRewardCC(app)
            if app.mode <= 1
                try
                    play(app.USsounds{app.netOC.myrandi(1,1,1,5)});
                end
                %pause(1);
            end
        end
        
        function  readRobotStimuli(app)
            % in the child class update the variable app.CCstimuli which is a vector
            % of dimension app.par.nrCCStimuli,1
            %
            % in che child class update the variable app.OCstimuli which is a vector
            % of dimensikon app.par.nrOStimuli,1
        end
        
        function startCC(app)
            app.learningMode = 1;
            app.trackingStimulus = -1;
        end
        
        function startOC(app)
            
            % small pause
            %pause(1);
            % say can you help me learn the colours?
            %playblocking(app.CSsounds{3});
            try
            playblocking(app.CSsounds{3});
            end
            
            % enter OC mode
            app.learningMode = 2;
        end
        
        function stopRunning(app)
        end
        
        function robotRewardOC(app)
        end
        
        function robotPunishmentOC(app)
        end
        
        function startTrackingTutor(app)
        end
        
        function touched = istouched(app)
            if app.touched %written by key listener
                touched = 1;
                disp('received touched');
                app.touched = 0; %reset
                app.stepsToReward = 0;
            else
                touched = 0;
            end
        end
        
        function punished = ispunished(app)
            if app.punished %written by key listener
                punished = 1;
                disp('received touched');
                app.punished = 0; %reset
                app.stepsToReward = 0;
            else
                punished = 0;
            end
        end
        
        function keypressed(app, src, evt)
            %disp(evt.Key);
            if strcmp(evt.Key, 't')
                app.touched = 1;
                disp('touched!');
            end
            if strcmp(evt.Key,'p')
                app.punished = 1;
                disp('punished');
            end
            if strcmp(evt.Key,'r')
                app.currentOCStimulus = 1;
                disp('OC-S1 is percieved');
            end
            if strcmp(evt.Key,'g')
                app.currentOCStimulus = 2;
                disp('OC-S2 is percieved');
            end
            for i= 1:9
                if strcmp(evt.Key,num2str(i))
                    app.newStimulus = i;
                    disp(['CC-S' num2str(i) ' is perceived']);
                end
            end
        end
        
        function createWindow(app)
            app.fig.panel = figure('Position',[0 0 1200 800]);
          %  set(app.fig.panel,'MenuBar','none');
            set(app.fig.panel,'Units','normalized');
            set(app.fig.panel,'Name','Output');
            
            app.fig.CCpanel = uipanel(app.fig.panel,'Position',[0 0.25 0.5 0.75]);
            app.fig.OCpanel = uipanel(app.fig.panel,'Position',[0.5 0.25 0.5 0.75]);
            app.fig.genPanel = uipanel(app.fig.panel,'Position',[0 0 1 0.25]);
            
            app.fig.CCsp1 = subplot('Position',[0.07 0.7 0.4 0.25],'Parent',app.fig.CCpanel);
            app.fig.CCsp3 = subplot('Position',[0.07 0.4 0.4 0.25],'Parent',app.fig.CCpanel);
            app.fig.CCsp5 = subplot('Position',[0.07 0.1 0.4 0.25],'Parent',app.fig.CCpanel,...
                'XTickLabel',{'0','200'},'XTick',[1 1000]);
            
            
            app.fig.CCsp2 = subplot('Position',[0.58 0.7 0.4 0.25],'Parent',app.fig.CCpanel);
            app.fig.CCsp4 = subplot('Position',[0.58 0.4 0.4 0.25],'Parent',app.fig.CCpanel);
            app.fig.CCsp6 = subplot('Position',[0.58 0.1 0.4 0.25],'Parent',app.fig.CCpanel);
            
            
            app.fig.OCsp1 = subplot('Position',[0.07 0.7 0.4 0.25],'Parent',app.fig.OCpanel);
            app.fig.OCsp3 = subplot('Position',[0.07 0.4 0.4 0.25],'Parent',app.fig.OCpanel);
            app.fig.OCsp5 = subplot('Position',[0.07 0.1 0.4 0.25],'Parent',app.fig.OCpanel);
            
            app.fig.OCsp2 = subplot('Position',[0.58 0.7 0.4 0.25],'Parent',app.fig.OCpanel);
            app.fig.OCsp4 = subplot('Position',[0.58 0.4 0.4 0.25],'Parent',app.fig.OCpanel);
            app.fig.OCsp6 = subplot('Position',[0.58 0.1 0.4 0.25],'Parent',app.fig.OCpanel);
            
            % gen panel
            % CC input
            app.fig.CCsp7 = subplot('Position',[0.07 0.1 0.25 0.8],'Parent',app.fig.genPanel);
            % real time window
            app.fig.CCsp9 = subplot('Position',[0.38 0.1 0.25 0.8],'Parent',app.fig.genPanel);
            % OC input
            app.fig.OCsp7 = subplot('Position',[0.70 0.1 0.25 0.8],'Parent',app.fig.genPanel);
            
            app.runbutton = uicontrol('Style', 'togglebutton',...
                'String','Start/Stop','Min',0,'Max',1,...
                'Parent',app.fig.panel,'Units','normalized','Position', [0 0 0.1 0.05],...
                'FontSize', 12,'Callback',@app.running);
            
        end
        
        function refreshGraphs(app,counter)
            
            if app.learningMode == 1
                set(app.fig.CCpanel,'Visible','on');
                set(app.fig.OCpanel,'Visible','off');
                switch mod(counter,8)
                    case 0,
                        plot(app.netCC.log.synapsesS(1:(app.netCC.currentStep-1),:),'Parent',app.fig.CCsp1);
                        set(app.fig.CCsp1,'XLim',[1 app.netCC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.CCsp1,'Sx to O');
                    case 1,
                        plot(app.netCC.log.modulation(1:(app.netCC.currentStep-1)),'Parent',app.fig.CCsp2);
                        set(app.fig.CCsp2,'XLim',[1 app.netCC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.CCsp2,'Modulation');
                    case 2,
                        plot([app.netCC.log.thresholdCor' app.netCC.log.thresholdDec'],'Parent',app.fig.CCsp3);
                        set(app.fig.CCsp3,'XLim',[1 app.netCC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.CCsp3,'Threshold');
                    case 3,
                        plot(app.netCC.log.activityOG(1:(app.netCC.currentStep-1)),'Parent',app.fig.CCsp4);
                        set(app.fig.CCsp4,'XLim',[1 app.netCC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.CCsp4,'Output Group');
                    case 4,
                        leftx = max(1, app.netCC.currentStep - 999);
                        plot([app.netCC.log.RCrate(leftx:(app.netCC.currentStep-1))' app.netCC.log.RDrate(leftx:(app.netCC.currentStep-1))'],...
                            'Parent',app.fig.CCsp5);
                        set(app.fig.CCsp5,'YGrid','on');
                        ylabel(app.fig.CCsp5,'RC rate');
                    case 5,
                        leftx = max(1, app.netCC.currentStep - 999);
                        plot(app.netCC.log.stimulusToTrack(leftx:(app.netCC.currentStep-1)),'Parent',app.fig.CCsp6);
                        set(app.fig.CCsp6,'XLim',[1 1000],'YGrid','on');
%                        set(app.fig.CCsp6,'XLim',[1 app.netCC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.CCsp6,'Stimulus to Track');
                    case 6,
                        leftx = max(1, app.netCC.currentStep - 499);
                        plot(app.netCC.log.stimuliRaw(leftx:app.netCC.currentStep-1,:),'Parent',app.fig.CCsp7,'Marker','d','Line','--');
                        set(app.fig.CCsp7,'Xlim',[1 500]);
                        set(app.fig.CCsp7,'Ylim',[0.5 app.netCC.par.nrStimuli + 0.5]);
                        set(app.fig.CCsp7','Color',[0 0 0],'YGrid','on','YColor',[0.5 0.5 0.5]);
                        ylabel(app.fig.CCsp7,'Raw inputs');
                    case 7,
                        plot(app.log.idleTime,'Parent',app.fig.CCsp9);
                        set(app.fig.CCsp9,'XLim',[1 app.currentStep],'YGrid','on','YLim',[-0.1 app.par.samplingTime]);
                        ylabel(app.fig.CCsp9,'Real time left');
                end
            end
            
            if app.learningMode == 2
                set(app.fig.OCpanel,'Visible','on');
                
                switch mod(counter,8)
                    case 0,
                        plot(app.netOC.log.synapsesStoO{1}(1:(app.netOC.currentStep-1),:),'Parent',app.fig.OCsp1);
                        set(app.fig.OCsp1,'XLim',[1 app.netOC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.OCsp1,'S1 to Ox');
                        
                    case 1,
                        fromRandomInput =1;
                        if app.par.nrOCStimuli <= 3
                            fromRandomInput =2;
                        end
                        if app.par.nrOCStimuli > 3
                            fromRandomInput = app.netOC.myrandi(1,1,1,app.par.nrOCStimuli);
                        end
                        
                        plot(app.netOC.log.synapsesStoO{fromRandomInput}(1:(app.netOC.currentStep-1),:),'Parent',app.fig.OCsp2);
                        set(app.fig.OCsp2,'XLim',[1 app.netOC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.OCsp2,['S ' num2str(fromRandomInput) ' to Ox']);
                    case 2,
                        plot([app.netOC.log.thresholdCor' app.netOC.log.thresholdDec'],'Parent',app.fig.OCsp3);
                        set(app.fig.OCsp3,'XLim',[1 app.netOC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.OCsp3,'Threshold');
                    case 3,
                        plot(app.netOC.log.activityOG,'Parent',app.fig.OCsp4);
                        set(app.fig.OCsp4,'XLim',[1 app.netOC.currentStep-1],'YGrid','on');
                        set(app.fig.OCsp4,'Xlim',[max(1, app.netOC.currentStep - 200) app.netOC.currentStep]);
                        ylabel(app.fig.OCsp4,'Output Group');
                    case 4,
                        plot(app.netOC.log.modulation(1:(app.netOC.currentStep-1)),'Parent',app.fig.OCsp5);
                        set(app.fig.OCsp5,'Xlim',[max(1, app.netOC.currentStep - 200) app.netOC.currentStep]);
                        set(app.fig.OCsp5,'XLim',[1 app.netOC.currentStep-1],'YGrid','on');
                        ylabel(app.fig.OCsp5,'Modulation');
                    case 5,
                        plot(app.netOC.log.actions,'Parent',app.fig.OCsp6,'Line','none','Marker','.');
                        set(app.fig.OCsp6,'Xlim',[max(1, app.netOC.currentStep - 200) app.netOC.currentStep]);
                        set(app.fig.OCsp6,'Ylim',[0.5 app.netOC.par.nrOutputs + 0.5]);
                        set(app.fig.OCsp6','Color',[0 0 0],'YGrid','on','YColor',[0.5 0.5 0.5]);
                        ylabel(app.fig.OCsp6,'Action');
                    case 6,
                        %leftx = max(1, app.netCC.currentStep - 999);
                        %plot([app.netOC.log.RCrate(leftx:(app.netOC.currentStep-1))' app.netOC.log.RDrate(leftx:(app.netOC.currentStep-1))'],...
                        %    'Parent',app.fig.OCsp7);
                        %set(app.fig.OCsp7,'YGrid','on');
                        %ylabel(app.fig.OCsp7,'RC rate');

                        plot(app.netOC.log.stimuliRaw(:,:),'Parent',app.fig.OCsp7,'Marker','d','Line','--');
                        set(app.fig.OCsp7,'Xlim',[max(1, app.netOC.currentStep - 200) app.netOC.currentStep]);
                        set(app.fig.OCsp7,'Ylim',[0.5 app.netOC.par.nrStimuli + 0.5]);
                        set(app.fig.OCsp7','Color',[0 0 0],'YGrid','on','YColor',[0.5 0.5 0.5]);
                    case 7,
                        plot(app.log.idleTime,'Parent',app.fig.CCsp9);
                        set(app.fig.CCsp9,'XLim',[1 app.currentStep],'YGrid','on','YLim',[-0.1 app.par.samplingTime]);
                        ylabel(app.fig.CCsp9,'Real time left');
                end
            end
            drawnow;
        end
    end
end
